import argparse
import os
from os import path as ph
from itertools import chain, groupby
from collections import defaultdict
import datetime
import shutil
import logging
import logging.handlers as hndlr
import json
import sys
import csv
import numbers

from fmtapper_web.config_utils import get_parser

logger = logging.getLogger("cleanup")

DEBUG = True


def files(root):
    for dirpath, dirnames, filenames in os.walk(root):
        for filename in filenames:
            yield ph.join(dirpath, filename)


def getsize(filename):
    try:
        return ph.getsize(filename)
    except:
        logger.error("Could not access file \"%s\"", filename)
        return 0


def size(files):
    sizes = map(getsize, files)
    return sum(sizes)


def size_mb(files):
    return size(files) / 1024.0 / 1024.0


def group_size(dirs):
    all_files = chain(*[files(d) for d in dirs])
    full_size = size(all_files)
    return full_size / 1024.0 / 1024.0


def getmtime(filename):
    mtime = ph.getmtime(filename)
    return datetime.datetime.fromtimestamp(mtime)


def is_older(dt, filename):
    mtime = getmtime(filename)
    now = datetime.datetime.now()
    return mtime + dt < now


def get_expired_dt(dt, filename):
    mtime = getmtime(filename)
    now = datetime.datetime.now()
    return now - mtime - dt


def removedirs(dir):
    try:
        os.removedirs(dir)
    except:
        logger.debug('Directory not empty')


def remove(filename):
    try:
        if not DEBUG:
            os.remove(filename)
            removedirs(os.path.dirname(filename))
    except:
        logger.error('Could not delete "%s"', filename)


def get_cleanuplog_ct(cleanup_log):
    with open(cleanup_log) as log:
        log.readline()
        line = log.readline()
        log.close()
        try:
            ctime = line.split('\t')[0]
            ctime = datetime.datetime.strptime(ctime, "%Y-%m-%d %H:%M:%S,%f")
            return ctime
        except:
            return datetime.datetime.now()


def self_cleanup(max_age):
    cleanup_log = '/var/log/fmtapper/cleanup.log'
    if os.path.isfile(cleanup_log):
        try:
            max_age = max_age_to_dt(max_age)
        except:
            return None
        cleanup_ct = get_cleanuplog_ct(cleanup_log)
        now = datetime.datetime.now()
        if cleanup_ct + max_age < now:
            msg = 'File "{0}" is expired for {1} and was deleted.'.format(cleanup_log, now - cleanup_ct - max_age)
            os.remove(cleanup_log)
            return msg
    return None


def max_age_to_dt(max_age):
    if not isinstance(max_age, datetime.timedelta):
        if not isinstance(max_age, numbers.Number):
            max_age = float(max_age)
        return datetime.timedelta(seconds=max_age)


def cleanup_directory(dirname, max_age):
    try:
        max_age = max_age_to_dt(max_age)
    except:
        logger.error("Incorrect max age for directory \"%s\"", dirname)
        return
    _files = list(files(dirname))
    _files = sorted(_files, key=getmtime)
    to_delete = filter(lambda x: is_older(max_age, x), _files)
    for filename in to_delete:
        logger.info('File "{0}" is expired for {1}'.format(filename, get_expired_dt(max_age, filename)))
        logger.info('Deleting "%s"', filename)
        remove(filename)


def mdatetime(filename):
    t = os.path.getmtime(filename)
    return datetime.datetime.fromtimestamp(t)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--config', type=str, default='cleanup_service.conf')
    parser.add_argument('--logging-config', nargs='?', type=str, default=None)
    parser.add_argument('--logs-dir', nargs='?', type=str, default="/var/log/fmtapper")
    parser.add_argument('--debug', action='store_true', default=False)
    args = parser.parse_args()

    parser = get_parser(args.config).parser
    store_logs = parser.getint('global', 'store-logs')
    store_logs_max_age = int(store_logs) * 24 * 60 * 60
    cleanup_log_size = size_mb(['/var/log/fmtapper/cleanup.log'])
    self_cleanup_msg = self_cleanup(store_logs_max_age)

    DEBUG = args.debug
    if DEBUG:
        logging.basicConfig(filename=ph.join(args.logs_dir, "cleanup.log"),
                            level=logging.DEBUG, format='%(asctime)s\t%(levelname)s\t%(message)s')
    else:
        logging.basicConfig(filename=ph.join(args.logs_dir, "cleanup.log"),
                            level=logging.INFO, format='%(asctime)s\t%(levelname)s\t%(message)s')

    if args.logging_config:
        logging.config.fileConfig(args.logging_config)
    else:
        logging.basicConfig()

    logger.info('Started')

    if self_cleanup_msg is not None:
        logger.info(self_cleanup_msg)
        cleanup_log_size = 0

    try:
        os.mkdir(args.logs_dir)
    except:
        pass

    tuners = [tuner for tuner in parser.sections() if "global" != tuner]
    groups = defaultdict(list)
    max_age = {}
    for tuner in tuners:
        logging.info("Reading tuner %s", tuner)
        audio_dir = parser.get(tuner, "audio-dir")
        logging.info("Audio dir = %s", audio_dir)
        groups["audio"].append(audio_dir)
        _max_age = parser.get(tuner, "store-audio")
        _max_age = sum((int(n) * mul for n, mul in zip(_max_age.split(':'), [24 * 60 * 60, 60 * 60, 60])))
        max_age[audio_dir] = _max_age
        log_dir = parser.get(tuner, "log-dir")
        logging.info("Log dir = %s", log_dir)
        groups["tuner-logs"].append(log_dir)
        max_age[log_dir] = int(parser.get(tuner, "store-logs")) * 24 * 60 * 60

    to_process = []

    for group, dirs_list in groups.iteritems():
        dirs = list(dirs_list)
        full_size = group_size(dirs) + (cleanup_log_size if group == "tuner-logs" else 0)
        target_size = parser.getint('global', group + '-max-weight')
        logger.info('Size of "%s" directories: %d MB', group, full_size)
        logger.info('Target size is %d MB', target_size)
        to_process.append((group, dirs))
        if full_size > target_size:
            logger.info('Size exceeds limit for %d MB', full_size - target_size)

    for group, dirs in to_process:
        logger.info('Cleaning up "%s" directories', group)
        for dirname in dirs:
            cleanup_directory(dirname, max_age[dirname])
        full_size = group_size(dirs) + (cleanup_log_size if group == "tuner-logs" else 0)
        target_size = parser.getint('global', group + '-max-weight')
        logger.info('Size of "%s" directories after cleaning up: %d MB', group, full_size)
        if full_size > target_size:
            logger.info('Size still exceeds limit for %d MB, deleting old files', full_size - target_size)
            all_files = chain(*[files(d) for d in dirs])
            all_files = sorted(all_files, key=ph.getmtime, reverse=True)
            while full_size > target_size and all_files:
                filename = all_files.pop()
                logger.warning('Deleting "%s"', filename)
                remove(filename)
                full_size = size_mb(all_files) + (cleanup_log_size if group == "tuner-logs" else 0)
            logger.info('Size of "%s" directories after cleaning up: %d MB', group, full_size)

    logger.info('Finished')
