# -*- coding: utf-8 -*-

import os
import re
from flask import current_app
import datetime
from fmtapper_web import config_utils
from json_utils import JSONIterator
from errors import ConfigError
import time


class Schedule(JSONIterator):
    def __init__(self, id, intervals, is_changes_applied=True, **kwargs):
        self.id = id
        self.intervals = intervals
        self.is_changes_applied = is_changes_applied

    def to_json(self):
        rv = dict()
        rv['id'] = self.id
        rv['intervals'] = map(lambda i: i.to_json(), self.intervals)
        rv['is_changes_applied'] = self.is_changes_applied
        return rv

    def save_to_file(self, config_dir, tuner_id):
        logger = current_app.logger
        logger.debug('Saving schedule %s to file', self.id)
        if not os.path.exists(config_dir):
            logger.debug('Creating config directory for schedules: %s', config_dir)
            os.mkdir(config_dir)
        elif not os.path.isdir(config_dir):
            raise ConfigError('Schedules config directory is not directory: %s', config_dir)

        file_name = self.id + '.sch'
        file_path = os.path.join(config_dir, file_name)
        if os.path.isfile(file_path):
            logger.debug('Schedule file exists: %s', file_path)
        else:
            logger.debug('Creating schedule file: %s', file_path)

        with open(file_path, 'w') as fp:
            lines = map(lambda i: i.format() + '\n', self.intervals)
            if len(lines) > 0:
                lines[-1] = lines[-1][:-1]
            fp.writelines(lines)

    @staticmethod
    def load_from_file(file_path, tuner_id):
        file_name = os.path.basename(file_path)
        id = file_name[0:-4]
        intervals = []
        with open(file_path) as f:
            for line in f:
                if line and line.strip() and not line.startswith('#'):
                    intervals.append(Interval.parse(line))

        is_changes_applied = Schedule.is_schedule_applied(id, tuner_id)
        return Schedule(id, intervals, is_changes_applied)

    @staticmethod
    def is_schedule_applied(sch_id, tuner_id):
        schedules_file = config_utils.get_schedules_applied_file_path(tuner_id)
        if os.path.isfile(schedules_file):
            with open(schedules_file, 'r') as fp:
                schedules = fp.readlines()
                return sch_id + '\n' in schedules

    @staticmethod
    def from_dict(data):
        schedule = Schedule(**data)
        if 'intervals' in data:
            intervals = map(lambda i: Interval(**i), data['intervals'])
        else:
            intervals = []
        schedule.intervals = intervals
        schedule.is_changes_applied = False
        return schedule

    def set_restart(self, old_schedule):
        self_json = self.to_json()
        old_schedule_json = old_schedule.to_json()
        del self_json['is_changes_applied']
        del old_schedule_json['is_changes_applied']
        self.is_changes_applied = self_json == old_schedule_json

    def set_applied_file(self, tuner_id):
        tmp_dir = config_utils.get_tmp_dir(tuner_id)
        if not os.path.exists(tmp_dir):
            os.makedirs(tmp_dir)

        if self.is_changes_applied:
            self.write_id_to_applied_file(tuner_id)
        else:
            self.remove_id_from_applied_file(tuner_id)

    def write_id_to_applied_file(self, tuner_id):
        schedules_applied = config_utils.get_schedules_applied_file_path(tuner_id)
        write_mode = 'a' if os.path.isfile(schedules_applied) else 'w'
        with open(schedules_applied, write_mode) as fp:
            fp.write(self.id + '\n')

    def remove_id_from_applied_file(self, tuner_id):
        schedules_file = config_utils.get_schedules_applied_file_path(tuner_id)
        applied_schedules = Schedules.get_applied_schedules_list(tuner_id)
        with open(schedules_file, 'w') as fp:
            applied_schedules_new = filter(lambda x: x.strip('\n') != self.id, applied_schedules)
            for sch in applied_schedules_new:
                fp.write(sch)


class Interval(JSONIterator):
    def __init__(self, start, end, **kwargs):
        self.start = start
        self.end = end

    def to_json(self):
        rv = dict()
        rv['start'] = self.start
        rv['end'] = self.end
        return rv

    def format(self):
        return "%s %s" % (Interval.seconds_to_timedelta(self.start), Interval.seconds_to_timedelta(self.end))

    def compare(self, other):
        if self.start < other.start:
            return -1
        elif self.start > other.start:
            return 1
        elif self.end < other.end:
            return -1
        elif self.end > other.end:
            return 1
        else:
            return 0

    def is_overlaps(self, other):
        return (other.start <= self.start < other.end) or (other.start < self.end <= other.end)

    @staticmethod
    def parse(str_interval):
        """Принимает на вход строку в формате "999 999" и возвращает соответствующий интервал.

        :param str_interval: строка
        :return: интервал
        """
        m = re.search('(\d+:\d+:\d+)\s+(\d+:\d+:\d+)', str_interval)
        if not m:
            raise ConfigError('Invalid schedule interval format: "%s"' % str_interval)
        start = int(Interval.timedelta_to_seconds(m.group(1)))
        end = int(Interval.timedelta_to_seconds(m.group(2)))
        if start >= end:
            raise ConfigError('Invalid schedule interval. start must be less than end: "%s"' % str_interval)
        if start < 0 or end < 0:
            raise ConfigError('Invalid schedule interval. start and end must not be negative: "%s"' % str_interval)
        day_seconds = 24 * 60 * 60
        if end > day_seconds:
            raise ConfigError('Invalid schedule interval. Invalid number of seconds: "%s"' % str_interval)
        return Interval(start, end)

    @staticmethod
    def timedelta_to_seconds(time_delta):
        try:
            x = time.strptime(time_delta, '%H:%M:%S')
        except ValueError:
            time_delta = time_delta.replace('24', '23')
            x = time.strptime(time_delta, '%H:%M:%S')
            return datetime.timedelta(hours=x.tm_hour+1, minutes=x.tm_min, seconds=x.tm_sec).total_seconds()
        return datetime.timedelta(hours=x.tm_hour, minutes=x.tm_min, seconds=x.tm_sec).total_seconds()

    @staticmethod
    def seconds_to_timedelta(sec):
        return '{:02}:{:02}:{:02}'.format(sec // 3600, sec % 3600 // 60, sec % 60)

    @staticmethod
    def comparator(a, b):
        return a.compare(b)

    @staticmethod
    def is_overlapped_intervals(intervals):
        """Проверяет, что данные интервалы не пересекаются

        :param intervals: список интервалов
        :return: True если пересекаются
        """
        intervals_sorted = sorted(intervals, Interval.comparator, reverse=True)
        full = None
        for i in intervals_sorted:
            if full is None:
                full = i
            elif full.is_overlaps(i):
                return True
            else:
                full = Interval(full.start, i.end)
        return False


class Schedules(JSONIterator):
    SCHEDULE_WEEK_NAMES = ['sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday']
    SCHEDULE_DEFAULT_NAME = "default"

    def __init__(self, tuner_id, default_sch=False, week_sch=[], date_sch=[]):
        self.default_sch = default_sch
        self.week_sch = week_sch
        self.date_sch = date_sch
        self.applied_count = self.applied_count(tuner_id)
        self.is_applied = self.is_schedules_applied(tuner_id)

    def to_json(self):
        rv = dict()
        rv['default_sch'] = self.default_sch
        rv['week_sch'] = self.week_sch
        rv['date_sch'] = self.date_sch
        rv['is_empty'] = self.is_empty()
        rv['applied_count'] = self.applied_count
        rv['is_applied'] = self.is_applied
        return rv

    def is_empty(self):
        return self.default_sch is False and self.week_sch == [] and self.date_sch == []

    @staticmethod
    def applied_count(tuner_id):
        return len(Schedules.get_applied_schedules_list(tuner_id))

    @staticmethod
    def is_schedules_applied(tuner_id):
        all_schedules_count = len(Schedules.get_schedules_list(tuner_id))
        applied_count = Schedules.applied_count(tuner_id)
        return applied_count == all_schedules_count

    @staticmethod
    def get_schedules_list(tuner_id):
        config_dir = config_utils.get_schedules_dir(tuner_id)
        schedules = []
        for (dirpath, dirnames, filenames) in os.walk(config_dir):
            schedules.extend(filenames)
        return map(lambda x: x[0:-4], schedules)

    @staticmethod
    def get_applied_schedules_list(tuner_id):
        schedules_file = config_utils.get_schedules_applied_file_path(tuner_id)
        if os.path.isfile(schedules_file):
            with open(schedules_file, 'r') as fp:
                return fp.readlines()
        else:
            return []
