from ConfigParser import ConfigParser
from lftp_mirror import get_ftp_status
from json_utils import JSONIterator
from config_utils import FakeGlobalSectionHead
import config_utils
from fmtapper_web.errors import ConfigError
import os
from flask import current_app


class DeviceConfig(JSONIterator):
    def __init__(self, name, sample_rate, dev_type=None, tuner=None, **kwargs):
        self.name = name
        self.sample_rate = sample_rate
        self.tuner = tuner
        self.dev_type = dev_type

    def to_json(self):
        rv = dict()
        rv['name'] = self.name
        rv['sample_rate'] = self.sample_rate
        rv['dev_type'] = self.dev_type
        rv['tuner'] = self.tuner
        return rv


class AudioDeviceConfig(JSONIterator):
    def __init__(self, name, sample_rate, tuner=None, **kwargs):
        self.name = name
        self.sample_rate = sample_rate
        self.type = Tuner.DeviceType.AUDIO_DEVICE
        self.tuner = tuner

    def to_json(self):
        rv = dict()
        rv['name'] = self.name
        rv['sample_rate'] = self.sample_rate
        rv['type'] = self.type
        rv['tuner'] = self.tuner
        return rv

    def write_config(self, parser):
        sec = 'global'
        parser.set(sec, 'audio-device', self.name)
        parser.set(sec, 'ad-sample-rate', self.sample_rate)

    def write_profile_config(self, parser):
        self.write_config(parser)

    @staticmethod
    def load_config(parser):
        sec = 'global'
        name = parser.get(sec, 'audio-device')
        sample_rate = parser.getint(sec, 'ad-sample-rate')
        return AudioDeviceConfig(name, sample_rate)


class FmTunerConfig(JSONIterator):
    def __init__(self, host, port, frequency, udp_port=5100, tuner_sample_rate=96000, tuner=None, **kwargs):
        self.host = host
        self.port = port
        self.frequency = frequency
        self.type = Tuner.DeviceType.FM_TUNER
        self.tuner = tuner
        self.udp_port = udp_port
        self.tuner_sample_rate = tuner_sample_rate

    def to_json(self):
        rv = dict()
        rv['host'] = self.host
        rv['port'] = self.port
        rv['type'] = self.type
        rv['tuner'] = self.tuner
        rv['frequency'] = self.frequency
        rv['udp_port'] = self.udp_port
        rv['tuner_sample_rate'] = self.tuner_sample_rate
        return rv

    def write_config(self, parser):
        sec = 'global'
        parser.set(sec, 'tuner-host', self.host)
        parser.set(sec, 'tuner-port', self.port)
        parser.set(sec, 'frequency', self.frequency)
        parser.set(sec, 'udp-port', self.udp_port)
        parser.set(sec, 'tuner-sample-rate', self.tuner_sample_rate)

    def write_profile_config(self, parser):
        self.write_config(parser)

    @staticmethod
    def load_config(parser):
        sec = 'global'
        host = parser.get(sec, 'tuner-host')
        port = parser.getint(sec, 'tuner-port')
        frequency = parser.getint(sec, 'frequency')
        udp_port = parser.getint(sec, 'udp-port', optional=True)
        tuner_sample_rate = parser.getint(sec, 'tuner-sample-rate', optional=True)
        return FmTunerConfig(host=host, port=port, frequency=frequency, udp_port=udp_port,
                             tuner_sample_rate=tuner_sample_rate)


class USBTunerConfig(JSONIterator):
    def __init__(self, name, sample_rate, frequency, tuner=None, radio_device_name=None, **kwargs):
        self.name = name
        self.sample_rate = sample_rate
        self.frequency = frequency
        self.type = Tuner.DeviceType.USB_TUNER
        self.tuner = tuner
        self.radio_device_name = radio_device_name

    def to_json(self):
        rv = dict()
        rv['name'] = self.name
        rv['sample_rate'] = self.sample_rate
        rv['frequency'] = self.frequency
        rv['type'] = self.type
        rv['tuner'] = self.tuner
        return rv

    def write_config(self, parser):
        sec = 'global'
        parser.set(sec, 'audio-device', self.name)
        parser.set(sec, 'ad-sample-rate', self.sample_rate)
        parser.set(sec, 'frequency', self.frequency)
        if self.radio_device_name is not None:
            parser.set(sec, 'radio-device-name', self.radio_device_name)

    def write_profile_config(self, parser):
        sec = 'global'
        parser.set(sec, 'audio-device', self.name)
        parser.set(sec, 'ad-sample-rate', self.sample_rate)
        parser.set(sec, 'frequency', self.frequency)

    @staticmethod
    def load_config(parser):
        sec = 'global'
        name = parser.get(sec, 'audio-device')
        radio_device_name = parser.get(sec, 'radio-device-name', optional=True)
        sample_rate = parser.getint(sec, 'ad-sample-rate')
        frequency = parser.getint(sec, 'frequency')
        return USBTunerConfig(name=name, radio_device_name=radio_device_name, sample_rate=sample_rate,
                              frequency=frequency)


class InetRadioTunerConfig(JSONIterator):
    def __init__(self, source_url, **kwargs):
        self.source_url = source_url
        self.type = Tuner.DeviceType.INET_RADIO

    def to_json(self):
        rv = dict()
        rv['source_url'] = self.source_url
        return rv

    def write_config(self, parser):
        sec = 'global'
        parser.set(sec, 'source-url', self.source_url)

    def write_profile_config(self, parser):
        self.write_config(parser)

    @staticmethod
    def load_config(parser):
        sec = 'global'
        source_url = parser.get(sec, 'source-url')
        return InetRadioTunerConfig(source_url)


class FtpConfig(JSONIterator):
    def __init__(self, ftp_url='', ftp_user='', ftp_pwd='', ftp_metadata=False, ftp_upload_interval='', **kwargs):
        self.ftp_url = ftp_url
        self.ftp_user = ftp_user
        self.ftp_pwd = ftp_pwd
        self.ftp_metadata = ftp_metadata
        self.ftp_upload_interval = ftp_upload_interval

    def to_json(self, tuner_id):
        rv = dict()
        rv['ftp_url'] = self.ftp_url
        rv['ftp_user'] = self.ftp_user
        rv['ftp_pwd'] = self.ftp_pwd
        rv['ftp_metadata'] = self.ftp_metadata
        rv['ftp_upload_interval'] = self.ftp_upload_interval
        rv['ftp_status'] = get_ftp_status(tuner_id)

        return rv

    def write_config(self, parser):
        sec = 'global'
        parser.set(sec, 'ftp-url', self.ftp_url)
        parser.set(sec, 'ftp-user', self.ftp_user)
        parser.set(sec, 'ftp-pwd', self.ftp_pwd)
        parser.set(sec, 'ftp-metadata', self.ftp_metadata)
        parser.set(sec, 'ftp-upload-interval', self.ftp_upload_interval)

    @staticmethod
    def load_config(parser):
        sec = 'global'

        ftp_url = parser.get(sec, 'ftp-url', value='', optional=True)
        ftp_user = parser.get(sec, 'ftp-user', value='', optional=True)
        ftp_pwd = parser.get(sec, 'ftp-pwd', value='', optional=True)
        ftp_metadata = parser.getboolean(sec, 'ftp-metadata', value=False, optional=True)
        ftp_upload_interval = parser.get(sec, 'ftp-upload-interval', value='', optional=True)

        return FtpConfig(ftp_url=ftp_url, ftp_user=ftp_user, ftp_pwd=ftp_pwd, ftp_metadata=ftp_metadata,
                         ftp_upload_interval=ftp_upload_interval)


class EncodingConfig(JSONIterator):
    def __init__(self, bit_rate=192000, sample_rate=44100, mono=False, url=None, interval_length=None,
                 fragment_length=None):
        self.bit_rate = bit_rate
        self.sample_rate = sample_rate
        self.mono = mono
        self.url = url
        self.interval_length = interval_length
        self.fragment_length = fragment_length

    def to_json(self):
        rv = dict()
        rv['bit_rate'] = self.bit_rate
        rv['sample_rate'] = self.sample_rate
        rv['mono'] = self.mono
        rv['url'] = self.url
        rv['interval_length'] = self.interval_length
        rv['fragment_length'] = self.fragment_length
        return rv

    def write_config(self, parser):
        sec = 'global'
        parser.set(sec, 'bit-rate', self.bit_rate)
        parser.set(sec, 'sample-rate', self.sample_rate)
        parser.set(sec, 'mono', +self.mono)
        parser.set(sec, 'url', self.url.encode('utf-8'))
        if self.interval_length:
            parser.set(sec, 'interval-length', self.interval_length)
        if self.fragment_length:
            parser.set(sec, 'fragment-length', self.fragment_length)

    @staticmethod
    def load_config(parser):
        sec = 'global'
        bit_rate = parser.getint(sec, 'bit-rate')
        sample_rate = parser.getint(sec, 'sample-rate')
        mono = parser.getboolean(sec, 'mono')
        url = parser.get(sec, 'url')
        interval_length = parser.getint(sec, 'interval-length', optional=True)
        fragment_length = parser.getint(sec, 'fragment-length', optional=True)
        return EncodingConfig(bit_rate=bit_rate, sample_rate=sample_rate, mono=mono, interval_length=interval_length,
                              fragment_length=fragment_length, url=url)


class FeedConfig(JSONIterator):
    def __init__(self, ice_genre=None, ice_name=None, ice_description=None, is_public=False, user_agent=None,
                 password='', content_type='', legacy_icecast=False):
        self.ice_genre = ice_genre
        self.ice_name = ice_name
        self.ice_description = ice_description
        self.is_public = is_public
        self.user_agent = user_agent
        self.password = password
        self.content_type = content_type
        self.legacy_icecast = legacy_icecast

    def to_json(self):
        rv = dict()
        rv['ice_genre'] = self.ice_genre
        rv['ice_name'] = self.ice_name
        rv['ice_description'] = self.ice_description
        rv['is_public'] = self.is_public
        rv['user_agent'] = self.user_agent
        rv['password'] = self.password
        rv['content_type'] = self.content_type
        rv['legacy_icecast'] = self.legacy_icecast
        return rv

    def write_config(self, file_path):
        logger = current_app.logger
        logger.debug('Save feed config to file %s', file_path)

        parser = ConfigParser()
        sec = 'global'
        parser.add_section(sec)
        parser.set(sec, 'password', self.password.encode('utf-8'))
        parser.set(sec, 'content_type', self.content_type)
        parser.set(sec, 'legacy_icecast', 1 if self.legacy_icecast else 0)
        parser.set(sec, 'is_public', 1 if self.is_public else 0)

        if self.ice_genre:
            parser.set(sec, 'ice_genre', self.ice_genre.encode('utf-8'))
        if self.ice_name:
            parser.set(sec, 'ice_name', self.ice_name.encode('utf-8'))
        if self.ice_description:
            parser.set(sec, 'ice_description', self.ice_description.encode('utf-8'))
        if self.user_agent:
            parser.set(sec, 'user_agent', self.user_agent.encode('utf-8'))

        with open(file_path, 'w') as fp:
            parser.write(FakeGlobalSectionHead(fp))

    @staticmethod
    def load_config(file_path):
        logger = current_app.logger
        logger.debug('Loading feed config from file %s', file_path)

        parser = config_utils.get_parser(file_path)
        sec = 'global'
        password = parser.get(sec, 'password')
        content_type = parser.get(sec, 'content_type')
        legacy_icecast = parser.getboolean(sec, 'legacy_icecast')
        is_public = parser.getboolean(sec, 'is_public')
        user_agent = parser.get(sec, 'user_agent', optional=True)
        ice_genre = parser.get(sec, 'ice_genre', optional=True)
        ice_name = parser.get(sec, 'ice_name', optional=True)
        ice_description = parser.get(sec, 'ice_description', optional=True)
        return FeedConfig(password=password, content_type=content_type, legacy_icecast=legacy_icecast,
                          is_public=is_public, user_agent=user_agent, ice_genre=ice_genre, ice_name=ice_name,
                          ice_description=ice_description)


class BroadcastConfig(JSONIterator):
    class BroadscastType:
        FFSERVER = 0
        ICECAST = 1

        def __init__(self):
            pass

    def __init__(self, server_type=BroadscastType.FFSERVER, feed_url=None, stream_url=None, feed_config=FeedConfig(), **kwargs):
        self.server_type = server_type
        self.feed_url = feed_url
        self.stream_url = stream_url
        self.feed_config = feed_config

    def to_json(self):
        rv = dict()
        rv['feed_url'] = self.feed_url
        rv['stream_url'] = self.stream_url
        rv['feed_config'] = self.feed_config.to_json()
        rv['server_type'] = self.server_type
        return rv

    def write_config(self, tuner_id, parser):
        logger = current_app.logger
        sec = 'global'
        if self.feed_url:
            parser.set(sec, 'feed-url', self.feed_url)
            parser.set(sec, 'stream-url', self.stream_url)
        parser.set(sec, 'server-type', self.server_type)

        feed_config_path = config_utils.get_feed_config_file_path(tuner_id)
        if self.server_type == BroadcastConfig.BroadscastType.ICECAST:
            logger.debug("Write feed config file %s", feed_config_path)
            self.feed_config.write_config(feed_config_path)
        elif os.path.isfile(feed_config_path):
            logger.debug('Delete feed config file %s', feed_config_path)
            try:
                os.remove(feed_config_path)
            except OSError:
                pass

    def write_profile(self, tuner_id, parser):
        sec = 'global'
        if self.feed_url:
            parser.set(sec, 'feed-url', self.feed_url)
        if self.server_type == BroadcastConfig.BroadscastType.ICECAST:
            feed_config_path = config_utils.get_feed_config_file_path(tuner_id)
            if os.path.isfile(feed_config_path):
                parser.set(sec, 'feed-config', feed_config_path)

    @staticmethod
    def load_config(tuner_id, parser):
        sec = 'global'
        feed_url = parser.get(sec, 'feed-url', optional=True, value=None)
        stream_url = parser.get(sec, 'stream-url', optional=True, value=None)
        server_type = parser.getint(sec, 'server-type', optional=True, value=0)
        if tuner_id != '':
            feed_config_path = config_utils.get_feed_config_file_path(tuner_id)
        else:
            feed_config_path = config_utils.get_tmp_feed_config_file_path()
        if os.path.isfile(feed_config_path):
            feed_config = FeedConfig.load_config(feed_config_path)
        else:
            feed_config = FeedConfig()

        return BroadcastConfig(server_type=server_type, feed_url=feed_url, stream_url=stream_url,
                               feed_config=feed_config)

    @staticmethod
    def from_dict(data):
        config = BroadcastConfig(**data)
        config.server_type = int(config.server_type)
        if 'feed_config' in data and data['feed_config'] is not None:
            config.feed_config = FeedConfig(**(data['feed_config']))
        return config


class Tuner(JSONIterator):
    class DeviceType:
        AUDIO_DEVICE = 0
        USB_TUNER = 1
        FM_TUNER = 2
        INET_RADIO = 3

        def __init__(self):
            pass

    class TunerStatus:
        RUNNING = 0
        STOP = 1
        UNKNOWN = 2

        def __init__(self):
            pass

    class WritingStatus:
        WRITING = 0
        DOES_NOT_WRITING = 1
        UNKNOWN = 2

        def __init__(self):
            pass

    class BroadcastStatus:
        BROADCASTING = 0
        DOES_NOT_BROADCASTING = 1
        UNKNOWN = 2

        def __init__(self):
            pass

    def __init__(self, id, name, channel_id=None, device_type=DeviceType.AUDIO_DEVICE,
                 device_config=None, encoding_config=EncodingConfig(), schedule=False, store_logs=None,
                 store_audio=None, log_dir=None, volume_window=5, broadcast_config=BroadcastConfig(),
                 runtime_info=None, ftp_config=FtpConfig(), is_changes_applied=True, is_schedules_empty=True):
        self.id = id
        self.name = name
        self.channel_id = channel_id
        self.device_type = device_type
        self.device_config = device_config
        self.store_logs = store_logs
        self.store_audio = store_audio
        self.encoding_config = encoding_config
        self.schedule = schedule
        self.log_dir = log_dir
        self.volume_window = volume_window
        self.runtime_info = runtime_info
        self.broadcast_config = broadcast_config
        self.ftp_config = ftp_config
        self.is_changes_applied = is_changes_applied
        self.is_schedules_empty = is_schedules_empty

    def to_json(self):
        rv = dict()
        rv['id'] = self.id
        rv['name'] = self.name
        rv['channel_id'] = self.channel_id
        rv['device_type'] = self.device_type
        rv['device_config'] = self.device_config.to_json()
        rv['encoding_config'] = self.encoding_config.to_json()
        from services import get_schedules
        is_schedules_empty = get_schedules(self.id).is_empty()
        rv['is_schedules_empty'] = is_schedules_empty
        rv['schedule'] = False if is_schedules_empty else self.schedule
        rv['store_logs'] = self.store_logs
        rv['store_audio'] = self.store_audio
        rv['log_dir'] = self.log_dir
        rv['volume_window'] = self.volume_window
        rv['runtime_info'] = self.runtime_info
        rv['broadcast_config'] = self.broadcast_config.to_json()
        rv['ftp_config'] = self.ftp_config.to_json(self.id)
        rv['is_changes_applied'] = self.is_changes_applied
        return rv

    def save_to_file(self, config_dir):
        logger = current_app.logger
        tuner_dir = os.path.join(config_dir, self.id)
        if not os.path.isdir(tuner_dir):
            logger.debug('Creating tuner directory %s', tuner_dir)
            os.mkdir(tuner_dir)
        logger.debug("broadcast: %s", self.broadcast_config)
        self.save_config_file()
        self.save_profile_file()

    def save_config_file(self):
        parser = ConfigParser()
        sec = 'global'
        parser.add_section(sec)
        parser.set(sec, 'name', self.name.encode('utf-8'))
        parser.set(sec, 'channel-id', self.channel_id)
        parser.set(sec, 'device-type', self.device_type)
        parser.set(sec, 'schedule', self.schedule)
        parser.set(sec, 'store-logs', self.store_logs)
        parser.set(sec, 'store-audio', self.store_audio)
        parser.set(sec, 'volume-window', self.volume_window)
        parser.set(sec, 'is-changes-applied', self.is_changes_applied)

        if self.encoding_config:
            self.encoding_config.write_config(parser)
        if self.device_config:
            self.device_config.write_config(parser)
        if self.broadcast_config:
            self.broadcast_config.write_config(self.id, parser)
        if self.ftp_config:
            self.ftp_config.write_config(parser)

        conf_file_path = config_utils.get_config_file_path(self.id)
        if not self.is_changes_applied:
            conf_file_path += '.new'

        with open(conf_file_path, 'w') as fp:
            parser.write(FakeGlobalSectionHead(fp))

    def save_profile_file(self):
        file_path = config_utils.get_profile_file_path(self.id)

        parser = ConfigParser()
        sec = 'global'
        parser.add_section(sec)
        parser.set(sec, 'channel-id', self.channel_id)
        parser.set(sec, 'logging-config', config_utils.get_tuner_logging_config_path(self.id))
        parser.set(sec, 'volume-window', self.volume_window)
        parser.set(sec, 'socket-file', config_utils.get_tuner_socket_file(self.id))
        if self.schedule:
            parser.set(sec, 'schedules', config_utils.get_schedules_dir(self.id))
        if self.encoding_config:
            self.encoding_config.write_config(parser)
        if self.device_config:
            self.device_config.write_profile_config(parser)
        if self.broadcast_config:
            self.broadcast_config.write_profile(self.id, parser)

        with open(file_path, 'w') as fp:
            parser.write(FakeGlobalSectionHead(fp))

    @staticmethod
    def from_dict(data):
        tuner = Tuner(**data)
        tuner.device_type = int(tuner.device_type)
        if 'encoding_config' in data:
            tuner.encoding_config = EncodingConfig(**(data['encoding_config']))
        if 'device_config' in data:
            if tuner.device_type == Tuner.DeviceType.AUDIO_DEVICE:
                tuner.device_config = AudioDeviceConfig(**(data['device_config']))
            elif tuner.device_type == Tuner.DeviceType.USB_TUNER:
                tuner.device_config = USBTunerConfig(**(data['device_config']))
            elif tuner.device_type == Tuner.DeviceType.FM_TUNER:
                tuner.device_config = FmTunerConfig(**(data['device_config']))
            else:
                tuner.device_config = InetRadioTunerConfig(**(data['device_config']))
        if 'broadcast_config' in data:
            tuner.broadcast_config = BroadcastConfig.from_dict(data['broadcast_config'])
        if 'ftp_config' in data:
            tuner.ftp_config = FtpConfig(**(data['ftp_config']))
        return tuner

    @staticmethod
    def load_from_file(tuner_id, runtime=False, from_archive=False):
        logger = current_app.logger

        if from_archive:
            file_path = os.path.join(config_utils.get_new_tuner_tmp_dir(), config_utils.TUNER_CONFIG_FILE_NAME)
        else:
            file_path = config_utils.get_config_file_path(tuner_id)

        if os.path.isfile(file_path+'.new') and not runtime:
            file_path += '.new'

        if os.path.isfile(file_path):
            logger.debug('Reading configuration from file %s', file_path)
        else:
            raise ConfigError('File %s not exists', file_path)

        parser = config_utils.get_parser(file_path)
        sec = 'global'
        name = parser.get(sec, 'name').decode('utf-8')
        channel_id = parser.get(sec, 'channel-id')
        schedule = parser.getboolean(sec, 'schedule')
        store_logs = parser.getint(sec, 'store-logs')
        store_audio = parser.get(sec, 'store-audio')
        volume_window = parser.getint(sec, 'volume-window', value=5)
        is_changes_applied = parser.getboolean(sec, 'is-changes-applied', value=True, optional=True)

        if from_archive:
            channel_id = ''

        encoding_config = EncodingConfig.load_config(parser)
        ftp_config = FtpConfig.load_config(parser)

        device_type = parser.getint(sec, 'device-type')
        if device_type == Tuner.DeviceType.AUDIO_DEVICE:
            device_config = AudioDeviceConfig.load_config(parser)
        elif device_type == Tuner.DeviceType.USB_TUNER:
            device_config = USBTunerConfig.load_config(parser)
        elif device_type == Tuner.DeviceType.FM_TUNER:
            device_config = FmTunerConfig.load_config(parser)
        else:
            device_config = InetRadioTunerConfig.load_config(parser)

        broadcast_config = BroadcastConfig.load_config(tuner_id, parser)

        tuner = Tuner(id=tuner_id, name=name, channel_id=channel_id, device_type=device_type,
                      device_config=device_config, encoding_config=encoding_config,
                      schedule=schedule, store_logs=store_logs, store_audio=store_audio, volume_window=volume_window,
                      broadcast_config=broadcast_config, ftp_config=ftp_config, is_changes_applied=is_changes_applied)
        return tuner

    def set_restart(self, other):
        self_json = self.to_json()
        other_json = other.to_json()

        del self_json['store_logs']
        del self_json['store_audio']
        del self_json['ftp_config']
        del self_json['is_changes_applied']
        del self_json['runtime_info']
        del self_json['broadcast_config']['stream_url']

        del other_json['store_logs']
        del other_json['store_audio']
        del other_json['ftp_config']
        del other_json['is_changes_applied']
        del other_json['runtime_info']
        del other_json['broadcast_config']['stream_url']

        self.is_changes_applied = (self_json == other_json)


class RuntimeInfo(JSONIterator):

    def __init__(self, tuner_id, status=Tuner.TunerStatus.UNKNOWN, err_num=None,
                 is_broadcasting=Tuner.BroadcastStatus.UNKNOWN, is_writing=Tuner.WritingStatus.UNKNOWN,
                 err_str=None, quality=-1, quality_abs=0, volume=-1, volume_abs=-90, ftp_status='', is_schedules_applied=False, applied_schedules_count=0):
        self.tuner_id = tuner_id
        self.status = status
        self.err_num = err_num
        self.is_broadcasting = is_broadcasting
        self.is_writing = is_writing
        self.err_str = err_str
        self.quality = quality
        self.quality_abs = quality_abs
        self.volume = volume
        self.volume_abs = volume_abs
        self.ftp_status = ftp_status
        self.is_schedules_applied = is_schedules_applied
        self.applied_schedules_count = applied_schedules_count
        t = Tuner.load_from_file(tuner_id, runtime=True)
        self.tuner_name = t.name
        self.tuner_device_type = t.device_type
        if isinstance(t.device_config, FmTunerConfig) or isinstance(t.device_config, USBTunerConfig):
            self.tuner_frequency = t.device_config.frequency
        else:
            self.tuner_frequency = None
        self.encoding_config = t.encoding_config

    def to_json(self):
        rv = dict()
        rv['tuner_id'] = self.tuner_id
        rv['status'] = self.status
        rv['err_num'] = self.err_num
        rv['is_writing'] = self.is_writing
        rv['is_broadcasting'] = self.is_broadcasting
        rv['err_str'] = self.err_str
        rv['quality'] = self.quality
        rv['quality_abs'] = self.quality_abs
        rv['volume'] = self.volume
        rv['volume_abs'] = self.volume_abs
        rv['ftp_status'] = self.ftp_status
        rv['is_schedules_applied'] = self.is_schedules_applied
        rv['applied_schedules_count'] = self.applied_schedules_count
        rv['tuner_name'] = self.tuner_name
        rv['tuner_device_type'] = self.tuner_device_type
        rv['tuner_frequency'] = self.tuner_frequency
        rv['encoding_config'] = self.encoding_config
        return rv
