# -*- coding: utf-8 -*-

from flask import current_app
from jinja2 import Environment, PackageLoader
import os
import subprocess
import re
from tuner import Tuner
import config_utils
import utils
from lftp_mirror import get_ftp_status


env = Environment(loader=PackageLoader("fmtapper_web", "templates"))


def gen_init_script(tuner, conf_path):
    logger = current_app.logger
    script_name = get_script_name(tuner.id)
    script_path = get_script_path(tuner.id)
    template = env.get_template('init_script.tpl')
    wrapper_script_path = current_app.config['WRAPPER_SCRIPT_PATH']
    args = dict()
    args['tuner_id'] = tuner.id
    args['conf_file_path'] = conf_path
    args['wrapper_script_path'] = wrapper_script_path
    args['logdir'] = config_utils.get_tuner_logs_dir(tuner.id)
    if tuner.device_type == Tuner.DeviceType.USB_TUNER:
        args['device'] = utils.get_radio_device_name(tuner.device_config.name)
        args['frequency'] = float(tuner.device_config.frequency) / 1000
    content = template.render(**args)
    logger.debug('Writing init script %s', script_name)
    with open(script_path, 'w') as f:
        f.writelines(content)


def del_init_script(tuner_id):
    script_path = get_script_path(tuner_id)
    if os.path.isfile(script_path):
        current_app.logger.debug(
            "Deleting init script %s", get_script_name(tuner_id))
        os.remove(script_path)
    else:
        current_app.logger.warn(
            "Init script %s does't exist", get_script_name(tuner_id))


def start_tuner(tuner_id):
    service_name = get_service_name(tuner_id)
    current_app.logger.info("Starting service %s", service_name)
    try:
        os.remove(get_stopped_file_path(tuner_id))
    except:
        pass
    return subprocess.call("start " + service_name, shell=True)


def stop_tuner(tuner_id):
    service_name = get_service_name(tuner_id)
    current_app.logger.info("Stopping service %s", service_name)
    file(get_stopped_file_path(tuner_id), 'w')
    return subprocess.call("stop " + service_name, shell=True)


def get_stopped_file_path(tuner_id):
    profile_file_path = config_utils.get_tuner_dir(tuner_id)
    return os.path.join(profile_file_path, 'stopped')


def restart_tuner(tuner_id):
    try:
        stop_tuner(tuner_id)
    finally:
        return start_tuner(tuner_id)


def get_tuner_status(tuner_id):
    logger = current_app.logger
    logger.debug('Checking service status for %s', tuner_id)
    p = subprocess.Popen(['status', get_service_name(
        tuner_id)], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    retval = p.wait()
    if retval > 0:
        logger.error("Can't get service status for %s", tuner_id)
        status = Tuner.TunerStatus.UNKNOWN
    else:
        status_str = ' '.join(p.stdout.readlines())
        if re.search('start/running', status_str):
            status = Tuner.TunerStatus.RUNNING
        else:
            status = Tuner.TunerStatus.STOP

    state_file = get_state_file_path(tuner_id, config_utils.get_state_dir())
    err_num = None
    err_str = None
    if os.path.isfile(state_file):
        with open(state_file, 'r') as f:
            err_num = f.readline().rstrip()
        if err_num is not None:
            err_num = int(err_num)
            err_str = os.strerror(err_num)
    logger.info('Service status is %s, error is %s for %s',
                status, err_num, tuner_id)
    ftp_status = get_ftp_status(tuner_id)
    return status, err_num, err_str, ftp_status


def get_service_name(tuner_id):
    return "fmtapper_" + tuner_id


def get_script_name(tuner_id):
    return "fmtapper_%s.conf" % tuner_id


def get_script_path(tuner_id):
    return os.path.join('/etc/init', get_script_name(tuner_id))


def get_state_file_path(tuner_id, state_dir):
    return os.path.join(state_dir, tuner_id + '.state')


def gen_logging_config(tuner_id):
    logger = current_app.logger
    file_path = config_utils.get_tuner_logging_config_path(tuner_id)
    template = env.get_template('logging_config.tpl')
    args = dict()
    args['path'] = config_utils.get_tuner_logs_dir(
        tuner_id) + '/%Y/%m/%d/fmtapper_%H%M%S%3N.log'

    content = template.render(**args)
    logger.debug('Writing logging config %s', file_path)
    with open(file_path, 'w') as f:
        f.writelines(content)
